//=============================================================================
// Error Handler
// by McKathlin
// MCK_ErrorHandler.js
// Last Update: 2015.11.07
//=============================================================================


/*:
 * @plugindesc Provides options for handling non-critical errors.
 * @author McKathlin
 *
 * @param Log Path
 * @desc The path of the error log file, relative to the project root.
 * @default error_log.txt
 *
 * @param On Audio Error
 * @desc What do do when there's an audio error.
 * Pop-up, Log, and/or Crash. Leave blank to ignore this error.
 * @default Pop-up, Log
 *
 * @param On Battleback Error
 * @desc What to do when a battleback file won't load.
 * Pop-up, Log, and/or Crash. Leave blank to ignore this error.
 * @default Pop-up, Log
 * 
 * @help This plugin does not provide plugin commands.
 *
 * The Error Handler plugin's purpose is to provide options for reporting
 * non-critical errors, without having to crash the game.
 *
 * There are no known conflicts with this plugin.
 * It is probably usable anywhere in the list. Near the end is recommended.
 * 
 * Each parameter named after the pattern "On ______ Error" accepts any
 * combination of the following three values: Log, Pop-up, Crash
 *        Log:  Write the error message to the log file.
 *     Pop-up:  Display a pop-up window with the error message.
 *      Crash:  Show the error message on the game screen, and end play.
 * These terms are case-insensitive. Terms other than these are ignored.
 * If none of these three recognized parameter values are given,
 * then the Error Handler plugin will cause that error type to be ignored.
 *
 */


var Imported = Imported || {};
Imported.MCK_ErrorHandler = true;


var McKathlin = McKathlin || {};
McKathlin.ErrorHandlerPlugin = McKathlin.ErrorHandlerPlugin || {};


//-----------------------------------------------------------------------------
/**
 * The Logger class.
 *
 * @class McKathlin.Log
 * @constructor
 * @param {String} filePath The path where the log should be written.
 */
 McKathlin.Log = function(filePath) {
    var fs = require('fs');
    var root = window.location.pathname.replace(/(\/www|)\/[^\/]*$/, '/');
    if (root.match(/^\/([A-Z]\:)/)) {
        root = root.slice(1);
    }
    rootPath = decodeURIComponent(root);
    var fullPath = rootPath + filePath;
    this._writeStream = fs.createWriteStream(fullPath,
        { flags: 'a' });
 };


 McKathlin.Log.prototype.write = function(str) {
    this._writeStream.write(str + "\r\n");
 };


 McKathlin.Log.prototype.close = function() {
    this._writeStream.end();
 };


//-----------------------------------------------------------------------------
/**
 * The ErrorHandler class.
 *
 * @class McKathlin.ErrorHandler
 * @constructor
 * @param {String} paramString The plugin parameter value: Log, PopUp, and/or Crash.
 * @param {McKathlin.Log} log Where to log the errors.
 */
McKathlin.ErrorHandler = function(paramString, log) {
    /**
     * @property logger
     * @type McKathlin.Log
     * @default null
     */
    this.logger = log;
    
    /**
     * @property doesLog
     * @type Boolean
     * @default false
     */
    this.doesLog = /log/i.test(paramString);
    
    /**
     * @property doesPopUp
     * @type Boolean
     * @default false
     */
    this.doesPopUp = /pop[\- ]?up/i.test(paramString);
    
    /**
     * @property doesCrash
     * @type Boolean
     * @default false
     */
    this.doesCrash = /crash/i.test(paramString);
};


McKathlin.ErrorHandler.prototype.handle = function(errorMessage) {
    if (this.doesLog && this.logger) {
        this.logger.write(errorMessage);
    }
    
    if (this.doesPopUp) {
        window.alert(errorMessage);
    }
    
    if (this.doesCrash) {
        throw new Error(errorMessage);
    }
};




//-----------------------------------------------------------------------------
// Parameter and Tool Init


McKathlin.Parameters = PluginManager.parameters('MCK_ErrorHandler');
McKathlin.Param = McKathlin.Param || {};


McKathlin.Param.OnAudioError = String(
    McKathlin.Parameters['On Audio Error']);
McKathlin.Param.OnBattlebackError = String(
    McKathlin.Parameters['On Battleback Error']);
McKathlin.Param.LogPath = String(McKathlin.Parameters['Log Path'])


McKathlin.ErrorLog = new McKathlin.Log(McKathlin.Param.LogPath);


McKathlin.AudioErrorHandler = new McKathlin.ErrorHandler(
    McKathlin.Param.OnAudioError, McKathlin.ErrorLog);
McKathlin.BattlebackErrorHandler = new McKathlin.ErrorHandler(
    McKathlin.Param.OnBattlebackError, McKathlin.ErrorLog);


//-----------------------------------------------------------------------------
// Audio Error Management


AudioManager.checkWebAudioError = function(webAudio) {
    if (webAudio && webAudio.isError()) {
        McKathlin.AudioErrorHandler.handle('Failed to load audio: ' + webAudio.url);
        webAudio.clear();
    }
};


//-----------------------------------------------------------------------------
// Image Error Management


// This replaces ImageManager.isReady() in rpg_managers.js
// Instead of crashing if a bitmap has an error,
// it calls a method to check for and handle bitmap errors.
ImageManager.isReady = function() {
    for (var key in this._cache) {
        var bitmap = this._cache[key];
        if (bitmap.isError()) {
            this.handleBitmapErrorAt(key);
        }
        if (!bitmap.isReady()) {
            return false;
        }
    }
    return true;
};


// New method to handle bitmap errors
ImageManager.handleBitmapErrorAt = function(key) {
    var bitmap = this._cache[key];
    if (bitmap.isError()) {
        if (/^img\/battlebacks[12]\//.test(bitmap.url)) {
            // This is a BattleBackground file.
            McKathlin.BattlebackErrorHandler.handle(
                'Failed to load battle background: ' + bitmap.url);
            // We can see what's going on without any battle background,
            // so replace the error'd bitmap with a blank one.
            this._cache[key] = new Bitmap();
        }
        else {
            // Other image types are not anti-crashed.
            // Make a crash similar to the default.
            throw new Error('Failed to load: ' + bitmap.url);
        }
    }
};